% convgain_spatial.m
%
% David Hale, CIT <dhale@astro.caltech.edu>
%
% This script computes the conversion gain given two "filmstrip"
% light images.

beep on

maxreject=5;         % number of outlier rejection passes
Fstep=5;

fileok1=0;
while fileok1==0,
    filename1 = input('Enter file name 1 (<ENTER> for list): ', 's');
    if isempty(filename1)
        dir *.fits
        file=0;
    else
        fileok1=1;
        filename1=strcat(filename1,'.fits');
    end
end % input filename1 (without .FITS extension)

fileok2=0;
while fileok2==0,
    filename2 = input('Enter file name 2 (<ENTER> for list): ', 's');
    if isempty(filename2)
        dir *.fits
        file=0;
    else
        fileok2=1;
        filename2=strcat(filename2,'.fits');
    end
end % input filename2 (without .FITS extension)
    
if (fileok1)
    display('  reading input FITS file 1...');
    im1=fitsread(filename1);
end

if (fileok2)
    display('  reading input FITS file 2...');
    im2=fitsread(filename2);
end

if (size(im1) == size(im2))

    display('  computing variances...');
    
    lincol     = size(im1);
    rows       = lincol(2);
    cols       = rows;
    lines      = lincol(1);
    frames     = lines / cols;
    plotpoints = frames-1;

    x = zeros(frames-1,1);
    y = x;
    
    reset1 = im1(1:rows,1:cols);
    reset2 = im2(1:rows,1:cols);

    jj=1+rows;
    for ii = 1:frames-1
        % form vector from row x col matrix
        e1v = reshape((reset1-im1(jj:jj+rows-1, 1:cols)),1,rows*cols);
        e2v = reshape((reset2-im2(jj:jj+rows-1, 1:cols)),1,rows*cols);
        
        % outlier rejection
        for kk = 1:maxreject
            sigma1 = std(e1v); m1 = mean(e1v);
            sigma2 = std(e2v); m2 = mean(e2v);
            outliers1 = abs(e1v-m1)>3*sigma1; outliers2 = abs(e2v-m2)>3*sigma2;
            e1v(outliers1) = 999999; e1v(outliers2) = 999999;
            e1v = e1v(e1v~=999999);
    
            e2v(outliers1) = 999999; e2v(outliers2) = 999999; 
            e2v = e2v(e2v~=999999);
        end

        y(ii) = var(e2v-e1v);
        x(ii) = mean(e2v+e1v);
        jj = jj + rows;
    end % calculations here
    
    display('  fitting');
    p = polyfit(x,y,1)
    f = polyval(p,x);
   
    hold off
    plot(x,y,'m.');
    hold on
    plot(x,f,'k');
    xlabel('mean'); ylabel('variance');
    
    s=sprintf('y = %2.5x + %3.5f', p(1),p(2));
    legend('data',s,'Location','SouthEast')
    
    beep
    
    ycut = input('Enter Y cut-off (<ENTER> for none): ', 's');
    if ~isempty(ycut)
        ycut=str2num(ycut);
        ycut_ind = find(y < ycut);
        p=polyfit(x(ycut_ind),y(ycut_ind),1)
        f=polyval(p,x(ycut_ind));
        figure;
        plot(x(ycut_ind),y(ycut_ind),'r.');
        hold on
        plot(x(ycut_ind),f,'b');    
    end

    display('  done!');
else
    display('images are not the same size! aborting.');
    
beep
beep

end
