% noisetest.m
%
% David Hale, CIT <dhale@astro.caltech.edu>
%
% This script computes the noise vs. sampling freq for a "filmstrip"
% light image.

%clear all;
%close all;
beep on

display('noisetest v2.3, 2012-10-09')

% Fstep=5;
fileok=0;  % set TRUE after a valid filename is entered
while fileok==0,
    filename = input('Enter file name (<ENTER> for list): ', 's');
    if isempty(filename)
    dir *.fits
    file=0;
    else
        outfile=strcat(filename,'.txt');
        infile=strcat(filename,'.fits');
        fileok=1;
    end
end % input filename

% enter conversion gain, if known
G=input('Enter conversion gain, if known (<ENTER> for unity): ', 's');
if ~isempty(G)
    G=str2double(G);
else
    G=1.;
end

% enter mux glow, if known
input_glow=input('Enter mux glow [ADU/read], if known (<ENTER> for 0): ', 's');
if ~isempty(G)
    input_glow=str2double(input_glow);
else
    input_glow=0.;
end

% enter frame time, if known
frame_time=input('Enter frame time in msec, if known: ', 's');
if ~isempty(frame_time)
    frame_time=str2double(frame_time)/1000.;
else
    frame_time=1.;
end


% input max fowler depth (or use predefined set for making plots)
finput=input('Enter number of Fowler samples, e.g. [1 2 3...100], or 0 to enter frequencies: ');
if (isempty(finput))
    %Fnset  = [ 1 2 3 4 5 7 10 14 16 20 35 50 100 ];
    Fnset  = [ 1 2 4 8 16 32 64 128 250 500 1000 ];
    %Fnset  = [ 1 2 4 8 16 32 64 ];
else
    Fnset = [finput];
end

if (Fnset==0)
    freqinput=input('Enter frequencies in Hz, e.g. [1000 500 250...100 1]: ');
    if (isempty(freqinput))
        Freqset = [1000 500 300 100 50 30 10 5 3 1];
    else
        Freqset = [freqinput];
    end
    Fnset = round(1./(frame_time.*Freqset));
end
    
Fnsize = size(Fnset);
numFns = Fnsize(2);

maxdepth = max(Fnset);

if (fileok)
    display('  reading input FITS file...');
    im=fitsread(infile);
end

lincol    = size(im);
lines     = lincol(1);
true_rows = lincol(2);    % the actual number of rows (not subarray)

% input rows, cols, offsets for subarrays
rows=input('Enter number of rows for subarray (<ENTER> for full window): ', 's');
if ~isempty(rows)
    rows=str2double(rows);
else
    rows = lincol(2);
end
cols   = rows;          % assume a square readout region
display('columns = rows (assuming square region)');

offset=input('Enter offset for subarray (<ENTER> for zero): ', 's');
if ~isempty(offset)
    offset=str2double(offset);
else
    offset=0;
end

frames = lines / true_rows;  % again, assumes square readout region

display('  computing Fowler images...');

%rows=4;cols=4;
%offset=60;

% for 1 pixel in 2x2 window
%rows=1; cols=1;
%offset=1;

% create arrays ahead of time...
rms_noise    = zeros(rows, cols, maxdepth);
avg_noise    = zeros(4,numFns);
avg_signal   = zeros(2,numFns);
noise_v_rate = avg_noise;
noise_surf   = zeros(10, 2, numFns);

Fn = 1;  % the first Fowler number
k  = 1;

while (k) <= numFns,
    Fn = Fnset(k);                              % the current requested Fowler number
    Ns = floor(frames/Fn);                      % number of samples possible at this Fn
%   muxe = 0.0034 * Fn;                         % mux glow in e- (from TMT study)
    muxe = input_glow * Fn;                     % mux glow in e- per read
    
%   disp(['  computing Fowler depth ', int2str(Fn), ' with ', int2str(Ns), ' samples...']);
    fprintf('  computing Fowler depth %d (%.2f Hz) with %d samples\n', Fn, 1./(Fn*frame_time), Ns);
    fowler_avg = zeros(rows, cols, Ns);         % store Ns fowler frame averages
    fowler_sub = zeros(rows, cols, Ns-1);       % store Ns-1 fowler subtraction frames

    i = 1;
    for z = 1:Ns
      fowler_sum = zeros(rows,cols);            % zero-out temporary summation matrix
        for j = 1:Fn
          temp = G * im(offset+i:offset+i+rows-1, offset+1:offset+cols);    % apply gain
          %use this line for individual pixels
          %temp = G * im(9+i:9+i+rows-1, 1:4);  % apply gain 
          %temp = G * im(0+i:0+i,1:1);
          fowler_sum(:,:) = fowler_sum + temp;  % store sum in here
          i = offset+i + true_rows;             % skip to start of next frame
        end
        fowler_avg(:,:,z) = fowler_sum./Fn;     % divide by number of sums
            
        if (z>1)
            fowler_sub(:,:,z-1) = fowler_avg(:,:,z) - fowler_avg(:,:,z-1);
        end
    end

    rms_noise(:,:,Fn) = std(fowler_sub,0,3);
    avg_signal(1,k)   = Fn .* frame_time;
    avg_signal(2,k)   = mean(mean(mean(fowler_sub)));
    

    %sigma(k) = std(mean(rms_noise(:,:,Fn)));
    
    % avg_noise(1,:) will be a vector of Fowler Numbers and
    % avg_noise(2,:) will be a vector of calculated Median Noise values
    avg_noise(1,k) = Fn;
    %avg_noise(2,k) = median(median(rms_noise(:,:,Fn)));
    avg_noise(2,k) = mean(mean(rms_noise(:,:,Fn)));
    
    % as above but by time instead of Fowler Average
    %frame_time = rows*cols/100000;
    noise_v_rate(1,k) = 1./(Fn * frame_time);   % frame rate in Hz if frame_time in sec
    noise_v_rate(2,k) = avg_noise(2,k);
    noise_v_rate(3,k) = sqrt( noise_v_rate(2,k).^2 - muxe );
    

    k=k+1;
end

n1=length(noise_v_rate(2,:));   % get index of last noise sample
n2=n1-1;                        % and second-to-last

% new calculated "mux glow" to make last two data points equal:
display('optimized mux glow:');
mg = abs(noise_v_rate(2,n1).^2-noise_v_rate(2,n2).^2) ./ (Fnset(n1)-Fnset(n2))

for k=1:numFns
    noise_v_rate(4,k) = sqrt( noise_v_rate(2,k).^2 - mg * Fnset(k) );
end

noise_v_time      = noise_v_rate;
noise_v_time(1,:) = 1./noise_v_time(1,:);

% % output med noise vs. fowler
% avg_out = avg_noise;
% avg_out = avg_out';
% dlmwrite('avg_noise.txt', avg_out, 'precision', '%10.5f');

% output med noise vs. time
avg_out = noise_v_rate;
avg_out = avg_out';
dlmwrite(outfile, avg_out, 'precision', '%10.5f');

avg_pix = zeros(rows,cols);
std_pix = zeros(rows,cols);

if (Fnset==1)
    display('calculating pixel means, stddevs...');
        for i=1:rows
            for j=1:cols
            pixel=reshape(fowler_sub(i,j,:),1,Ns-1);
            avg_pix(i,j) = mean(pixel);
            std_pix(i,j) = std(pixel);
            end
        end

    display('high noise pixel coordinates:');
    [x,y] = find(std_pix >= floor(max(max(std_pix))))

    display('low noise pixel coordinates:');
    [x,y] = find(std_pix <= min(min(std_pix)))

    display('low dark current pixel coordinates:');
    [x,y] = find(avg_pix <= min(min(avg_pix)))

    display('high dark current pixel coordinates:');
    [x,y] = find(avg_pix >= floor(max(max(avg_pix))))
end

beep
beep
           
display('  done!');
