%% pixelgain.m
%% David Hale

beep on
display('pixelgain, 2013-09-24');

indirok=0;  % set TRUE after a valid directory is entered
while indirok==0,
    indirname = input('Enter directory name for mean, stddev images: ', 's');
    if isempty(indirname)
        indirok=0;
    else
        indirok=1;
    end
end % input directory name

% make sure there is a trailing backslash
s=size(indirname);
s=s(2);
if indirname(s) ~= '/'
    indirname=strcat(indirname,'/');
end

inok=0;
while inok==0,
    numfilmstrips=input('Enter number of filmstrips in file: ', 's');
    if isempty(numfilmstrips)
        inok=0;
    else
        inok=1;
    end
end
numfilmstrips=str2num(numfilmstrips);

numexpo = 2*numfilmstrips-3;

[v, s] = deal( zeros(2048,2048,numexpo) );

fprintf('reading variance files...\n');
for ii=1:numexpo
    fprintf(' %d...', ii);
    s(:,:,ii)=fitsread(strcat(indirname,'sigma_',int2str(ii),'.fits'));
    v(:,:,ii)=s(:,:,ii).^2;
end
clear s;

m = zeros(2048,2048,numexpo);

fprintf('\nreading mean files...\n');

for ii=1:numexpo
    fprintf(' %d...', ii);
    m(:,:,ii)=fitsread(strcat(indirname,'mean_',int2str(ii),'.fits'));
end

fprintf('done\n');

fit = zeros(2048,2048,numexpo);
rsq = zeros(2048,2048);
p = zeros(2048,2048,2);

fprintf('fitting row ');
for row=1:2048
    fprintf('%04d', row);
    for col=1:2048
        p(row,col,:) = polyfit(m(row,col,:), v(row,col,:), 1);
        fit(row,col,:) = polyval(squeeze(p(row,col,:)), m(row,col,:));
        resid = v(row,col,:) - fit(row,col,:);
        SSresid = sum(resid.^2);
        SStotal = (length(v(row,col,:))-1)*var(v(row,col,:));
        rsq(row,col) = 1 - SSresid/SStotal;
    end
    fprintf('\b\b\b\b');
end

fprintf('saving FITS files...\n');

%% for cooelab's Matlab:
% fits_write_image( strcat(indirname,'rsq.fits'),     rot90(fliplr(rsq(:,:)))    );
% fits_write_image( strcat(indirname,'gain.fits'),    rot90(fliplr(p(:,:,1)))    );
% fits_write_image( strcat(indirname,'invgain.fits') ,rot90(fliplr(1./p(:,:,1))) );

%% for lucius' Matlab:
fitswrite( rsq(:,:),    strcat(indirname, 'rsq.fits')     );
fitswrite( p(:,:,1),    strcat(indirname, 'gain.fits')    );  
fitswrite( 1./p(:,:,1), strcat(indirname, 'invgain.fits') );


fprintf('saving matlab variables...\n');
save( strcat(indirname,'pixelgain.mat'), 'm','v','fit','p','rsq' );

beep
beep

fprintf('done\n');
